@@ -1,37 +1,31 @@
 /*
- * This file is part of libtrace
  *
- * Copyright (c) 2007,2008,2009,2010 The University of Waikato, Hamilton, 
- * New Zealand.
- *
- * Authors: Daniel Lawson 
- *          Perry Lorier
- *          Shane Alcock 
- *          
+ * Copyright (c) 2007-2016 The University of Waikato, Hamilton, New Zealand.
  * All rights reserved.
  *
- * This code has been developed by the University of Waikato WAND 
+ * This file is part of libtrace.
+ *
+ * This code has been developed by the University of Waikato WAND
  * research group. For further information please see http://www.wand.net.nz/
  *
  * libtrace is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * it under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * libtrace is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with libtrace; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
- * $Id: format_legacy.c 1581 2010-05-05 02:56:42Z perry $
  *
  */
-
+#ifndef _GNU_SOURCE
 #define _GNU_SOURCE
+#endif
 
 #include "config.h"
 #include "common.h"
@@ -41,7 +35,6 @@
 #include "wandio.h"
 
 #include <sys/stat.h>
-#include <assert.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <stdio.h>
@@ -140,7 +133,6 @@ static time_t trtime(char *s) {
 	/* XXX: this function may not be particularly portable to
 	 * other platforms, e.g. *BSDs, Windows */
 	struct tm tm;
-	char *tz;
 	time_t ret;
 
 	if(sscanf(s, "%4u%2u%2u-%2u%2u%2u", &tm.tm_year, &tm.tm_mon,
@@ -154,7 +146,12 @@ static time_t trtime(char *s) {
 	tm.tm_yday = 0; /* ignored */
 	tm.tm_isdst = -1; /* forces check for summer time */
 	
-	tz = getenv("TZ");
+	/*
+	if (getenv("TZ") == NULL) {
+		fprintf(stderr, "Failed to get the current TZ");
+		return (time_t)0;
+	}
+	*/
 	if (putenv("TZ=Pacific/Auckland")) {
 		perror("putenv");
 		return (time_t)0;
@@ -261,7 +258,8 @@ static int legacy_read_packet(libtrace_t *libtrace, libtrace_packet_t *packet) {
 			packet->type = TRACE_RT_DATA_LEGACY_ETH;
 			break;
 		default:
-			assert(0);
+			trace_set_err(libtrace, TRACE_ERR_BAD_FORMAT, "Invalid trace format type in legacy_read_packet()");
+			return -1;
 	}
 
 	/* This is going to block until we either get an entire record
@@ -272,7 +270,7 @@ static int legacy_read_packet(libtrace_t *libtrace, libtrace_packet_t *packet) {
 						buffer,
 						(size_t)64)) != 64) {
 			if (numbytes < 0) {
-				trace_set_err(libtrace,errno,"read(%s)",libtrace->uridata);
+				trace_set_err(libtrace,TRACE_ERR_WANDIO_FAILED,"read(%s)",libtrace->uridata);
 			} else if (numbytes > 0) {
 				
 				continue;
@@ -313,7 +311,7 @@ static int legacynzix_read_packet(libtrace_t *libtrace, libtrace_packet_t *packe
 		if ((numbytes = wandio_read(libtrace->io, buffer,
 						(size_t)68)) != 68) {
 			if (numbytes < 0) {
-				trace_set_err(libtrace,errno,"read(%s)",libtrace->uridata);
+				trace_set_err(libtrace,TRACE_ERR_WANDIO_FAILED,"read(%s)",libtrace->uridata);
 			} else if (numbytes > 0)
 				continue;
 			return numbytes;
@@ -395,7 +393,12 @@ static int legacynzix_get_capture_length(const libtrace_packet_t *packet UNUSED)
 
 static int legacypos_get_wire_length(const libtrace_packet_t *packet) {
 	legacy_pos_t *lpos = (legacy_pos_t *)packet->header;
-	assert(ntohl(lpos->wlen)>0);
+
+	if (ntohl(lpos->wlen) <= 0) {
+		trace_set_err(packet->trace, TRACE_ERR_BAD_PACKET, "Packet wire length is invalid (%d) "
+			"in legacypos_get_wire_length()", ntohl(lpos->wlen));
+		return -1;
+	}
 	return ntohl(lpos->wlen);
 }
 
@@ -416,7 +419,7 @@ static int legacynzix_get_wire_length(const libtrace_packet_t *packet) {
 static uint64_t legacy_get_erf_timestamp(const libtrace_packet_t *packet)
 {
 	legacy_ether_t *legacy = (legacy_ether_t*)packet->header;
-	return legacy->ts;
+	return bswap_le_to_host64(legacy->ts);
 }  
 
 static uint32_t ts_cmp(uint32_t ts_a, uint32_t ts_b) {
@@ -439,8 +442,6 @@ static struct timeval legacynzix_get_timeval(const libtrace_packet_t *packet) {
 	struct timeval tv;
 	uint32_t hdr_ts;
 
-	double dts;
-	
 	legacy_nzix_t *legacy = (legacy_nzix_t *)packet->header;
 		
 	hdr_ts = legacy->ts;
@@ -460,13 +461,13 @@ static struct timeval legacynzix_get_timeval(const libtrace_packet_t *packet) {
 	DATA(packet->trace)->ts_high = new_ts;
 
 
-	dts = tv.tv_sec + (double)tv.tv_usec / 1000 / 1000;
+	/*dts = tv.tv_sec + (double)tv.tv_usec / 1000 / 1000; */
 	return tv;
 	
 }	
 
 static void legacypos_help(void) {
-	printf("legacypos format module: $Revision: 1581 $\n");
+	printf("legacypos format module: $Revision: 1753 $\n");
 	printf("Supported input URIs:\n");
 	printf("\tlegacypos:/path/to/file\t(uncompressed)\n");
 	printf("\tlegacypos:/path/to/file.gz\t(gzip-compressed)\n");
@@ -477,7 +478,7 @@ static void legacypos_help(void) {
 }
 
 static void legacyatm_help(void) {
-	printf("legacyatm format module: $Revision: 1581 $\n");
+	printf("legacyatm format module: $Revision: 1753 $\n");
 	printf("Supported input URIs:\n");
 	printf("\tlegacyatm:/path/to/file\t(uncompressed)\n");
 	printf("\tlegacyatm:/path/to/file.gz\t(gzip-compressed)\n");
@@ -488,7 +489,7 @@ static void legacyatm_help(void) {
 }
 
 static void legacyeth_help(void) {
-	printf("legacyeth format module: $Revision: 1581 $\n");
+	printf("legacyeth format module: $Revision: 1753 $\n");
 	printf("Supported input URIs:\n");
 	printf("\tlegacyeth:/path/to/file\t(uncompressed)\n");
 	printf("\tlegacyeth:/path/to/file.gz\t(gzip-compressed)\n");
@@ -499,7 +500,7 @@ static void legacyeth_help(void) {
 }
 
 static void legacynzix_help(void) {
-	printf("legacynzix format module: $Revision: 1581 $\n");
+	printf("legacynzix format module: $Revision: 1753 $\n");
 	printf("Supported input URIs:\n");
 	printf("\tlegacynzix:/path/to/file\t(uncompressed)\n");
 	printf("\tlegacynzix:/path/to/file.gz\t(gzip-compressed)\n");
@@ -511,7 +512,7 @@ static void legacynzix_help(void) {
 
 static struct libtrace_format_t legacyatm = {
 	"legacyatm",
-	"$Id: format_legacy.c 1581 2010-05-05 02:56:42Z perry $",
+	"$Id$",
 	TRACE_FORMAT_LEGACY_ATM,
 	NULL,				/* probe filename */
 	NULL,				/* probe magic */
@@ -527,7 +528,9 @@ static struct libtrace_format_t legacyatm = {
 	legacy_read_packet,		/* read_packet */
 	legacy_prepare_packet,		/* prepare_packet */
 	NULL,				/* fin_packet */
+        NULL,                           /* can_hold_packet */
 	NULL,				/* write_packet */
+	NULL,				/* flush_output */
 	legacyatm_get_link_type,	/* get_link_type */
 	NULL,				/* get_direction */
 	NULL,				/* set_direction */
@@ -535,6 +538,7 @@ static struct libtrace_format_t legacyatm = {
 	NULL,				/* get_timeval */
 	NULL,				/* get_timespec */
 	NULL,				/* get_seconds */
+	NULL,                           /* get_meta_section */
 	NULL,				/* seek_erf */
 	NULL,				/* seek_timeval */
 	NULL,				/* seek_seconds */
@@ -545,16 +549,17 @@ static struct libtrace_format_t legacyatm = {
 	NULL,				/* get_received_packets */
 	NULL,				/* get_filtered_packets */
 	NULL,				/* get_dropped_packets */
-	NULL,				/* get_captured_packets */
+	NULL,				/* get_statistics */
 	NULL,				/* get_fd */
 	trace_event_trace,		/* trace_event */
 	legacyatm_help,			/* help */
-	NULL				/* next pointer */
+	NULL,				/* next pointer */
+	NON_PARALLEL(false)
 };
 
 static struct libtrace_format_t legacyeth = {
 	"legacyeth",
-	"$Id: format_legacy.c 1581 2010-05-05 02:56:42Z perry $",
+	"$Id$",
 	TRACE_FORMAT_LEGACY_ETH,
 	NULL,				/* probe filename */
 	NULL,				/* probe magic */
@@ -570,7 +575,9 @@ static struct libtrace_format_t legacyeth = {
 	legacy_read_packet,		/* read_packet */
 	legacy_prepare_packet,		/* prepare_packet */
 	NULL,				/* fin_packet */
+        NULL,                           /* can_hold_packet */
 	NULL,				/* write_packet */
+	NULL,				/* flush_output */
 	legacyeth_get_link_type,	/* get_link_type */
 	NULL,				/* get_direction */
 	NULL,				/* set_direction */
@@ -578,6 +585,7 @@ static struct libtrace_format_t legacyeth = {
 	NULL,				/* get_timeval */
 	NULL,				/* get_timespec */
 	NULL,				/* get_seconds */
+	NULL,                           /* get_meta_section */
 	NULL,				/* seek_erf */
 	NULL,				/* seek_timeval */
 	NULL,				/* seek_seconds */
@@ -588,16 +596,17 @@ static struct libtrace_format_t legacyeth = {
 	NULL,				/* get_received_packets */
 	NULL,				/* get_filtered_packets */
 	NULL,				/* get_dropped_packets */
-	NULL,				/* get_captured_packets */
+	NULL,				/* get_statistics */
 	NULL,				/* get_fd */
 	trace_event_trace,		/* trace_event */
 	legacyeth_help,			/* help */
-	NULL				/* next pointer */
+	NULL,				/* next pointer */
+	NON_PARALLEL(false)
 };
 
 static struct libtrace_format_t legacypos = {
 	"legacypos",
-	"$Id: format_legacy.c 1581 2010-05-05 02:56:42Z perry $",
+	"$Id$",
 	TRACE_FORMAT_LEGACY_POS,
 	NULL,				/* probe filename */
 	NULL,				/* probe magic */
@@ -613,7 +622,9 @@ static struct libtrace_format_t legacypos = {
 	legacy_read_packet,		/* read_packet */
 	legacy_prepare_packet,		/* prepare_packet */
 	NULL,				/* fin_packet */
+        NULL,                           /* can_hold_packet */
 	NULL,				/* write_packet */
+	NULL,				/* flush_output */
 	legacypos_get_link_type,	/* get_link_type */
 	NULL,				/* get_direction */
 	NULL,				/* set_direction */
@@ -621,6 +632,7 @@ static struct libtrace_format_t legacypos = {
 	NULL,				/* get_timeval */
 	NULL,				/* get_timespec */
 	NULL,				/* get_seconds */
+	NULL,                           /* get_meta_section */
 	NULL,				/* seek_erf */
 	NULL,				/* seek_timeval */
 	NULL,				/* seek_seconds */
@@ -631,16 +643,17 @@ static struct libtrace_format_t legacypos = {
 	NULL,				/* get_received_packets */
 	NULL,				/* get_filtered_packets */
 	NULL,				/* get_dropped_packets */
-	NULL,				/* get_captured_packets */
+	NULL,				/* get_statistics */
 	NULL,				/* get_fd */
 	trace_event_trace,		/* trace_event */
 	legacypos_help,			/* help */
 	NULL,				/* next pointer */
+	NON_PARALLEL(false)
 };
 
 static struct libtrace_format_t legacynzix = {
 	"legacynzix",
-	"$Id: format_legacy.c 1581 2010-05-05 02:56:42Z perry $",
+	"$Id$",
 	TRACE_FORMAT_LEGACY_NZIX,
 	NULL,				/* probe filename */
 	NULL,				/* probe magic */
@@ -656,7 +669,9 @@ static struct libtrace_format_t legacynzix = {
 	legacynzix_read_packet,		/* read_packet */
 	legacy_prepare_packet,		/* prepare_packet */
 	NULL,				/* fin_packet */
+        NULL,                           /* can_hold_packet */
 	NULL,				/* write_packet */
+	NULL,				/* flush_output */
 	legacynzix_get_link_type,	/* get_link_type */
 	NULL,				/* get_direction */
 	NULL,				/* set_direction */
@@ -664,6 +679,7 @@ static struct libtrace_format_t legacynzix = {
 	legacynzix_get_timeval,		/* get_timeval */
 	NULL,				/* get_timespec */
 	NULL,				/* get_seconds */
+	NULL,                           /* get_meta_section */
 	NULL,				/* seek_erf */
 	NULL,				/* seek_timeval */
 	NULL,				/* seek_seconds */
@@ -674,11 +690,12 @@ static struct libtrace_format_t legacynzix = {
 	NULL,				/* get_received_packets */
 	NULL,				/* get_filtered_packets */
 	NULL,				/* get_dropped_packets */
-	NULL,				/* get_captured_packets */
+	NULL,				/* get_statistics */
 	NULL,				/* get_fd */
 	trace_event_trace,		/* trace_event */
 	legacynzix_help,		/* help */
 	NULL,				/* next pointer */
+	NON_PARALLEL(false)
 };
 	
 void legacy_constructor(void) {
