@@ -1,36 +1,28 @@
 /*
- * This file is part of libtrace
  *
- * Copyright (c) 2007,2008,2009,2010 The University of Waikato, Hamilton, 
- * New Zealand.
- *
- * Authors: Daniel Lawson 
- *          Perry Lorier
- *          Shane Alcock 
- *          
+ * Copyright (c) 2007-2016 The University of Waikato, Hamilton, New Zealand.
  * All rights reserved.
  *
- * This code has been developed by the University of Waikato WAND 
+ * This file is part of libtrace.
+ *
+ * This code has been developed by the University of Waikato WAND
  * research group. For further information please see http://www.wand.net.nz/
  *
  * libtrace is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * it under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * libtrace is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with libtrace; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
- * $Id: format_pcap.c 1664 2010-10-06 00:37:07Z salcock $
  *
  */
-
 #include "common.h"
 #include "config.h"
 #include "libtrace.h"
@@ -38,7 +30,6 @@
 #include "format_helper.h"
 
 #include <sys/stat.h>
-#include <assert.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -46,9 +37,6 @@
 
 #ifdef HAVE_PCAP_H
 #  include <pcap.h>
-#  ifdef HAVE_PCAP_INT_H
-#    include <pcap-int.h>
-#  endif
 #endif
 
 /* This format module deals with traces captured using the PCAP library. This
@@ -106,9 +94,31 @@ struct pcap_format_data_out_t {
 	} output;
 };
 
+static bool pcap_can_write(libtrace_packet_t *packet) {
+	/* Get the linktype */
+        libtrace_linktype_t ltype = trace_get_link_type(packet);
+
+        if (ltype == TRACE_TYPE_PCAPNG_META
+                || ltype == TRACE_TYPE_CONTENT_INVALID
+                || ltype == TRACE_TYPE_UNKNOWN
+                || ltype == TRACE_TYPE_ERF_META
+                || ltype == TRACE_TYPE_NONDATA) {
+
+                return false;
+        }
+
+        return true;
+}
+
 static int pcap_init_input(libtrace_t *libtrace) {
 	libtrace->format_data = malloc(sizeof(struct pcap_format_data_t));
 
+	if (!libtrace->format_data) {
+		trace_set_err(libtrace, TRACE_ERR_INIT_FAILED, "Unable to allocate memory for "
+			"format data inside pcap_init_input()");
+		return -1;
+	}
+
 	INPUT.pcap = NULL;
 	DATA(libtrace)->filter = NULL;
 	DATA(libtrace)->snaplen = LIBTRACE_PACKET_BUFSIZE;
@@ -135,6 +145,7 @@ static int pcap_start_input(libtrace_t *libtrace) {
 	}
 
 	/* If a filter has been configured, compile and apply it */
+#ifdef HAVE_BPF
 	if (DATA(libtrace)->filter) {
 		if (DATA(libtrace)->filter->flag == 0) {
 			pcap_compile(INPUT.pcap, 
@@ -150,6 +161,7 @@ static int pcap_start_input(libtrace_t *libtrace) {
 			return -1;
 		}
 	}
+#endif
 	return 0;
 }
 
@@ -159,8 +171,12 @@ static int pcap_config_input(libtrace_t *libtrace,
 {
 	switch(option) {
 		case TRACE_OPTION_FILTER:
+#ifdef HAVE_BPF
 			DATA(libtrace)->filter=data;
 			return 0;
+#else
+			return -1;
+#endif
 		case TRACE_OPTION_SNAPLEN:
 			/* Snapping isn't supported directly, so fall thru
 			 * and let libtrace deal with it
@@ -174,11 +190,17 @@ static int pcap_config_input(libtrace_t *libtrace,
 		default:
 			return -1;
 	}
-	assert(0);
+	return -1;
 }
 
 static int pcap_init_output(libtrace_out_t *libtrace) {
 	libtrace->format_data = malloc(sizeof(struct pcap_format_data_out_t));
+	if (!libtrace->format_data) {
+		trace_set_err_out(libtrace, TRACE_ERR_INIT_FAILED, "Unable to allocate memory for "
+			"format data inside pcap_init_output()");
+		return -1;
+	}
+
 	OUTPUT.trace.pcap = NULL;
 	OUTPUT.trace.dump = NULL;
 	return 0;
@@ -187,6 +209,12 @@ static int pcap_init_output(libtrace_out_t *libtrace) {
 static int pcapint_init_output(libtrace_out_t *libtrace) {
 #ifdef HAVE_PCAP_INJECT
 	libtrace->format_data = malloc(sizeof(struct pcap_format_data_out_t));
+	if (!libtrace->format_data) {
+                trace_set_err_out(libtrace, TRACE_ERR_INIT_FAILED, "Unable to allocate memory for "
+			"format data inside pcapint_init_output()");
+                return -1;
+        }
+
 	OUTPUT.trace.pcap = NULL;
 	OUTPUT.trace.dump = NULL;
 	return 0;
@@ -206,6 +234,12 @@ static int pcapint_init_output(libtrace_out_t *libtrace) {
 
 static int pcapint_init_input(libtrace_t *libtrace) {
 	libtrace->format_data = malloc(sizeof(struct pcap_format_data_t));
+	if (!libtrace->format_data) {
+		trace_set_err(libtrace, TRACE_ERR_INIT_FAILED, "Unable to allocate memory for "
+			"format data inside pcapint_init_input()");
+		return -1;
+	}
+
 	DATA(libtrace)->filter = NULL;
 	DATA(libtrace)->snaplen = LIBTRACE_PACKET_BUFSIZE;
 	DATA(libtrace)->promisc = 0;
@@ -218,8 +252,12 @@ static int pcapint_config_input(libtrace_t *libtrace,
 {
 	switch(option) {
 		case TRACE_OPTION_FILTER:
+#ifdef HAVE_BPF
 			DATA(libtrace)->filter=(libtrace_filter_t*)data;
 			return 0;
+#else
+			return -1;
+#endif
 		case TRACE_OPTION_SNAPLEN:
 			DATA(libtrace)->snaplen=*(int*)data;
 			return 0;
@@ -237,12 +275,59 @@ static int pcapint_config_input(libtrace_t *libtrace,
 			 * error-setting. */
 			return -1;
 	}
-	assert(0);
+	return -1;
 }
 
 static int pcapint_start_input(libtrace_t *libtrace) {
 	char errbuf[PCAP_ERRBUF_SIZE];
+
+#ifdef HAVE_PCAP_CREATE
+	int ret = 0;
 	
+	if ((INPUT.pcap = pcap_create(libtrace->uridata, errbuf)) == NULL) {
+		trace_set_err(libtrace,TRACE_ERR_INIT_FAILED,"%s",errbuf);
+		return -1; /* failure */
+	}
+	if ((pcap_set_snaplen(INPUT.pcap, DATA(libtrace)->snaplen) == 
+				PCAP_ERROR_ACTIVATED)) {
+		trace_set_err(libtrace,TRACE_ERR_INIT_FAILED,"%s",errbuf);
+		return -1; /* failure */
+	}
+
+	if ((pcap_set_promisc(INPUT.pcap, DATA(libtrace)->promisc) == 
+				PCAP_ERROR_ACTIVATED)) {
+		trace_set_err(libtrace,TRACE_ERR_INIT_FAILED,"%s",errbuf);
+		return -1; /* failure */
+	}
+	
+	if ((pcap_set_timeout(INPUT.pcap, 1) == PCAP_ERROR_ACTIVATED)) {
+		trace_set_err(libtrace,TRACE_ERR_INIT_FAILED,"%s",errbuf);
+		return -1; /* failure */
+	}
+
+#ifdef HAVE_PCAP_IMMEDIATE
+        if ((pcap_set_immediate_mode(INPUT.pcap, 1) == PCAP_ERROR_ACTIVATED)) {
+		trace_set_err(libtrace,TRACE_ERR_INIT_FAILED,"%s",errbuf);
+		return -1; /* failure */
+	}
+#endif
+
+	if ((ret = pcap_activate(INPUT.pcap)) != 0) {
+		if (ret == PCAP_WARNING_PROMISC_NOTSUP) {
+			trace_set_err(libtrace, TRACE_ERR_INIT_FAILED,"Promiscuous mode unsupported");
+			return -1;
+		}
+		if (ret == PCAP_WARNING) {
+			pcap_perror(INPUT.pcap, "Pcap Warning:");
+		} else {
+			trace_set_err(libtrace,TRACE_ERR_INIT_FAILED,"%s",
+					pcap_geterr(INPUT.pcap));
+			return -1;
+		}
+	}
+			 
+#else	
+
 	/* Open the live device */
 	if ((INPUT.pcap = 
 			pcap_open_live(libtrace->uridata,
@@ -253,31 +338,72 @@ static int pcapint_start_input(libtrace_t *libtrace) {
 		trace_set_err(libtrace,TRACE_ERR_INIT_FAILED,"%s",errbuf);
 		return -1; /* failure */
 	}
+#endif
+#ifdef HAVE_PCAP_SETNONBLOCK
+	pcap_setnonblock(INPUT.pcap,0,errbuf);
+#endif
 	/* Set a filter if one is defined */
+#ifdef HAVE_BPF
 	if (DATA(libtrace)->filter) {
-		if (pcap_setfilter(INPUT.pcap,&DATA(libtrace)->filter->filter)
+		struct pcap_pkthdr *pcap_hdr = NULL;
+		u_char *pcap_payload = NULL;
+                int pcapret;
+		
+                if (DATA(libtrace)->filter->flag == 0) {
+			pcap_compile(INPUT.pcap, 
+					&DATA(libtrace)->filter->filter,
+					DATA(libtrace)->filter->filterstring, 
+					1, 0);
+			DATA(libtrace)->filter->flag = 1;
+		}
+                if (pcap_setfilter(INPUT.pcap,&DATA(libtrace)->filter->filter)
 			== -1) {
 			trace_set_err(libtrace,TRACE_ERR_INIT_FAILED,"%s",
 					pcap_geterr(INPUT.pcap));
 			return -1; /* failure */
 		}
+
+                /* Consume the first packet in the queue, as this may not
+                 * have had the filter applied to it.
+                 *
+                 * Otherwise we can get problems with the event API, where
+                 * select tells us that there is a packet available but
+                 * calling trace_read_packet will block forever because the
+                 * packet in the queue didn't match the filter so
+                 * pcap_next_ex returns "timed out".
+                 *
+                 * This does mean we may consume a legitimate packet, but
+                 * that's a pretty small downside compared with trace_event
+                 * getting stuck in an infinite loop because of pcap
+                 * wackiness. 
+                 *
+                 * For some reason, we only need to consume one packet for
+                 * this to work, so let's hope that holds in the future.
+                 */
+                do {
+        		pcapret = pcap_next_ex(INPUT.pcap, &pcap_hdr, 
+				(const u_char **)&pcap_payload);
+		} while (0);
+
+                if (pcapret < 0)
+                        return -1;
 	}
-#ifdef HAVE_PCAP_SETNONBLOCK
-	pcap_setnonblock(INPUT.pcap,0,errbuf);
 #endif
 	return 0; /* success */
 }
 
-static int pcap_pause_input(libtrace_t *libtrace)
+static int pcap_pause_input(libtrace_t *libtrace UNUSED)
 {
-	pcap_close(INPUT.pcap);
-	INPUT.pcap=NULL;
 	return 0; /* success */
 }
 
 
 static int pcap_fin_input(libtrace_t *libtrace) 
 {
+	if (INPUT.pcap != NULL) {
+		pcap_close(INPUT.pcap);
+	}
+	INPUT.pcap=NULL;
 	free(libtrace->format_data);
 	return 0; /* success */
 }
@@ -288,7 +414,9 @@ static int pcap_fin_output(libtrace_out_t *libtrace)
 		pcap_dump_flush(OUTPUT.trace.dump);
 		pcap_dump_close(OUTPUT.trace.dump);
 	}
-	pcap_close(OUTPUT.trace.pcap);
+	if (OUTPUT.trace.pcap) {
+		pcap_close(OUTPUT.trace.pcap);
+	}
 	free(libtrace->format_data);
 	return 0;
 }
@@ -333,28 +461,21 @@ static int pcap_prepare_packet(libtrace_t *libtrace, libtrace_packet_t *packet,
 static int pcap_read_packet(libtrace_t *libtrace, libtrace_packet_t *packet) {
 	int ret = 0;
 	int linktype;
-	uint32_t flags = 0;
-	
-	assert(libtrace->format_data);
-	linktype = pcap_datalink(DATA(libtrace)->input.pcap);
-	packet->type = pcap_linktype_to_rt(linktype);
 
-	/* If we're using the replacement pcap_next_ex() we need to
-	 * make sure we have a buffer to *shudder* memcpy into 
-	 */
-	if (!packet->buffer) {
-		packet->buffer = malloc(LIBTRACE_PACKET_BUFSIZE);
-		if (!packet->buffer) {
-			trace_set_err(libtrace, errno, 
-					"Cannot allocate memory");
-			return -1;
-		}
-		packet->header = packet->buffer;
-		packet->payload = (char *)packet->buffer+sizeof(struct pcap_pkthdr);
-			
+	if (!libtrace->format_data) {
+		trace_set_err(libtrace, TRACE_ERR_BAD_FORMAT, "Trace format data missing, "
+			"call trace_create() before calling pcap_read_packet()");
+		return -1;
 	}
 
-	flags |= TRACE_PREP_OWN_BUFFER;
+        /* If the packet buffer is owned by libtrace free it */
+        if (packet->buf_control == TRACE_CTRL_PACKET) {
+                free(packet->buffer);
+                packet->buffer = NULL;
+        }
+
+	linktype = pcap_datalink(DATA(libtrace)->input.pcap);
+	packet->type = pcap_linktype_to_rt(linktype);
 	
 	for(;;) {
 		
@@ -364,12 +485,20 @@ static int pcap_read_packet(libtrace_t *libtrace, libtrace_packet_t *packet) {
 		ret = pcap_next_ex(INPUT.pcap, &pcap_hdr, 
 				(const u_char **)&pcap_payload);
 		
+		packet->buffer = pcap_hdr;
 		packet->header = pcap_hdr;
 		packet->payload = pcap_payload;
+		packet->buf_control = TRACE_CTRL_EXTERNAL;
 
 		switch(ret) {
 			case 1: break; /* no error */
-			case 0: continue; /* timeout expired */
+			case 0: 
+				if ((ret=is_halted(libtrace)) != -1)
+					return ret;
+                                /* timeout, return and let libtrace check message
+                                 * queue.
+                                 */
+                                return READ_MESSAGE;
 			case -1: 
 				trace_set_err(libtrace,TRACE_ERR_BAD_PACKET,
 						"%s",pcap_geterr(INPUT.pcap));
@@ -399,6 +528,21 @@ static int pcap_read_packet(libtrace_t *libtrace, libtrace_packet_t *packet) {
 static int pcap_write_packet(libtrace_out_t *libtrace, 
 		libtrace_packet_t *packet) 
 {
+
+	/* Check pcap can write this type of packet */
+        if (!pcap_can_write(packet)) {
+                return 0;
+        }
+
+	if (!libtrace) {
+		fprintf(stderr, "NULL trace passed into pcap_write_packet()\n");
+		return TRACE_ERR_NULL_TRACE;
+	}
+	if (!packet) {
+		trace_set_err_out(libtrace, TRACE_ERR_NULL_PACKET, "NULL packet passed into pcap_write_packet()\n");
+		return -1;
+	}
+
 	struct pcap_pkthdr pcap_pkt_hdr;
 	void *link;
 	libtrace_linktype_t linktype;
@@ -411,7 +555,7 @@ static int pcap_write_packet(libtrace_out_t *libtrace,
 	/* If this packet cannot be converted to a pcap linktype then
 	 * pop off the top header until it can be converted
 	 */
-	while (libtrace_to_pcap_linktype(linktype)==~0U) {
+	while (libtrace_to_pcap_linktype(linktype)==TRACE_DLT_ERROR) {
 		if (!demote_packet(packet)) {
 			trace_set_err_out(libtrace, 
 				TRACE_ERR_NO_CONVERSION,
@@ -477,21 +621,35 @@ static int pcap_write_packet(libtrace_out_t *libtrace,
 		else
 			pcap_pkt_hdr.len = trace_get_wire_length(packet);
 
-		assert(pcap_pkt_hdr.caplen<65536);
-		assert(pcap_pkt_hdr.len<65536);
+		if (pcap_pkt_hdr.caplen >= 65536) {
+			trace_set_err_out(libtrace, TRACE_ERR_BAD_HEADER, "Header capture length is larger than it should be in pcap_write_packet()");
+			return -1;
+		}
+		if (pcap_pkt_hdr.len >= 65536) {
+			trace_set_err_out(libtrace, TRACE_ERR_BAD_HEADER, "Header wire length is larger than it should be pcap_write_packet()");
+			return -1;
+		}
 
 		pcap_dump((u_char*)OUTPUT.trace.dump, &pcap_pkt_hdr, packet->payload);
 	}
-	return 0;
+	return remaining;
+}
+
+static int pcap_flush_output(libtrace_out_t *libtrace) {
+        return pcap_dump_flush(OUTPUT.trace.dump);
 }
 
 static int pcapint_write_packet(libtrace_out_t *libtrace,
 		libtrace_packet_t *packet) 
 {
 	int err;
+	libtrace_linktype_t linktype = trace_get_link_type(packet);
 
-	if (trace_get_link_type(packet) == TRACE_TYPE_NONDATA)
+	/* Silently discard RT metadata packets and packets with an
+	 * unknown linktype. */
+	if (linktype == TRACE_TYPE_NONDATA || linktype == TRACE_TYPE_UNKNOWN || linktype == TRACE_TYPE_ERF_META || linktype == TRACE_TYPE_CONTENT_INVALID) {
 		return 0;
+	}
 
 	if (!OUTPUT.trace.pcap) {
 		OUTPUT.trace.pcap = (pcap_t *)pcap_open_live(
@@ -526,7 +684,11 @@ static libtrace_linktype_t pcap_get_link_type(const libtrace_packet_t *packet) {
 
 static libtrace_direction_t pcap_set_direction(libtrace_packet_t *packet,
 		libtrace_direction_t dir) {
-	
+
+	/* We only support tagging with IN or OUT return error for any others */
+	if(!(dir == TRACE_DIR_OUTGOING || dir == TRACE_DIR_INCOMING))
+		return -1;
+
 	/* PCAP doesn't have a direction field in the header, so we need to
 	 * promote to Linux SLL to tag it properly */
 	libtrace_sll_header_t *sll;
@@ -544,67 +706,9 @@ static libtrace_direction_t pcap_set_direction(libtrace_packet_t *packet,
 	return dir;
 }
 
-static libtrace_direction_t pcap_get_direction(const libtrace_packet_t *packet) {
-	libtrace_direction_t direction  = -1;
-	switch(pcap_get_link_type(packet)) {
-		/* Only packets encapsulated in Linux SLL or PFLOG have any
-		 * direction information */
-
-		case TRACE_TYPE_LINUX_SLL:
-		{
-			libtrace_sll_header_t *sll;
-			sll = trace_get_packet_buffer(packet, NULL, NULL);
-			/* TODO: should check remaining>=sizeof(*sll) */
-			if (!sll) {
-				trace_set_err(packet->trace,
-					TRACE_ERR_BAD_PACKET,
-						"Bad or missing packet");
-				return -1;
-			}
-			/* 0 == LINUX_SLL_HOST */
-			/* the Waikato Capture point defines "packets
-			 * originating locally" (ie, outbound), with a
-			 * direction of 0, and "packets destined locally"
-			 * (ie, inbound), with a direction of 1.
-			 * This is kind-of-opposite to LINUX_SLL.
-			 * We return consistent values here, however
-			 *
-			 * Note that in recent versions of pcap, you can
-			 * use "inbound" and "outbound" on ppp in linux
-			 */
-			if (sll->pkttype == TRACE_SLL_OUTGOING) {
-				direction = TRACE_DIR_OUTGOING;
-			} else {
-				direction = TRACE_DIR_INCOMING;
-			}
-			break;
-
-		}
-		case TRACE_TYPE_PFLOG:
-		{
-			libtrace_pflog_header_t *pflog;
-			pflog = trace_get_packet_buffer(packet, NULL, NULL);
-			/* TODO: should check remaining >= sizeof(*pflog) */
-			if (!pflog) {
-				trace_set_err(packet->trace,
-						TRACE_ERR_BAD_PACKET,
-						"Bad or missing packet");
-				return -1;
-			}
-			/* enum    { PF_IN=0, PF_OUT=1 }; */
-			if (ntohs(pflog->dir==0)) {
-
-				direction = TRACE_DIR_INCOMING;
-			}
-			else {
-				direction = TRACE_DIR_OUTGOING;
-			}
-			break;
-		}
-		default:
-			break;
-	}	
-	return direction;
+static libtrace_direction_t pcapint_get_direction(const libtrace_packet_t *packet) {
+        /* This function is defined in format_helper.c */
+        return pcap_get_direction(packet);
 }
 
 
@@ -618,9 +722,16 @@ static struct timeval pcap_get_timeval(const libtrace_packet_t *packet) {
 
 
 static int pcap_get_capture_length(const libtrace_packet_t *packet) {
+	if (!packet) {
+		fprintf(stderr, "NULL packet passed into pcapng_get_capture_length()\n");
+		return TRACE_ERR_NULL_PACKET;
+	}
 	struct pcap_pkthdr *pcapptr = 0;
 	pcapptr = (struct pcap_pkthdr *)packet->header;
-	assert(pcapptr->caplen<=65536);
+	if (pcapptr->caplen > 65536) {
+		trace_set_err(packet->trace, TRACE_ERR_BAD_PACKET, "Capture length is to large, Packet may be corrupt in pcap_get_capture_length()");
+		return -1;
+	}
 
 	return pcapptr->caplen;
 }
@@ -652,40 +763,54 @@ static int pcap_get_framing_length(UNUSED const libtrace_packet_t *packet) {
 
 static size_t pcap_set_capture_length(libtrace_packet_t *packet,size_t size) {
 	struct pcap_pkthdr *pcapptr = 0;
-	assert(packet);
+	if (!packet) {
+		fprintf(stderr, "NULL packet passed to pcap_set_capture_length()\n");
+		return TRACE_ERR_NULL_PACKET;
+	}
 	if (size > trace_get_capture_length(packet)) {
 		/* Can't make a packet larger */
 		return trace_get_capture_length(packet);
 	}
 	/* Reset the cached capture length */
-	packet->capture_length = -1;
+	packet->cached.capture_length = -1;
 	pcapptr = (struct pcap_pkthdr *)packet->header;
 	pcapptr->caplen = size;
 	return trace_get_capture_length(packet);
 }
 
 static int pcap_get_fd(const libtrace_t *trace) {
-
-	assert(trace->format_data);
+	if (!trace) {
+		fprintf(stderr, "NULL trace passed to pcap_get_fd()\n");
+		return TRACE_ERR_NULL_TRACE;
+	}
+	if (!trace->format_data) {
+		/* cant do this because trace is a const? */
+		/*trace_set_err(trace, TRACE_ERR_BAD_FORMAT, "Trace format data missing, call init_input() before calling pcap_get_fd()");*/
+		fprintf(stderr, "Trace format data missing, call init_input() before calling pcap_get_fd()\n");
+		return TRACE_ERR_BAD_FORMAT;
+	}
 	return pcap_fileno(DATA(trace)->input.pcap);
 }
 
-static uint64_t pcap_get_dropped_packets(libtrace_t *trace)
-{
-	struct pcap_stat stats;
-	if (pcap_stats(DATA(trace)->input.pcap,&stats)==-1) {
+static void pcap_get_statistics(libtrace_t *trace, libtrace_stat_t *stat) {
+
+	struct pcap_stat pcapstats;
+	if (pcap_stats(DATA(trace)->input.pcap,&pcapstats)==-1) {
 		char *errmsg = pcap_geterr(DATA(trace)->input.pcap);
 		trace_set_err(trace,TRACE_ERR_UNSUPPORTED,
-				"Failed to retreive stats: %s\n",
+				"Failed to retrieve stats: %s\n",
 				errmsg ? errmsg : "Unknown pcap error");
-		return ~0;
+		return;
 	}
 
-	return stats.ps_drop;
+        stat->received_valid = 1;
+        stat->received = pcapstats.ps_recv;
+        stat->dropped_valid = 1;
+        stat->dropped = pcapstats.ps_drop;
 }
 
 static void pcap_help(void) {
-	printf("pcap format module: $Revision: 1664 $\n");
+	printf("pcap format module: $Revision: 1729 $\n");
 	printf("Supported input URIs:\n");
 	printf("\tpcap:/path/to/file\n");
 	printf("\n");
@@ -697,7 +822,7 @@ static void pcap_help(void) {
 }
 
 static void pcapint_help(void) {
-	printf("pcapint format module: $Revision: 1664 $\n");
+	printf("pcapint format module: $Revision: 1729 $\n");
 	printf("Supported input URIs:\n");
 	printf("\tpcapint:interface\n");
 	printf("\n");
@@ -711,14 +836,14 @@ static void pcapint_help(void) {
 
 static struct libtrace_format_t pcap = {
 	"pcap",
-	"$Id: format_pcap.c 1664 2010-10-06 00:37:07Z salcock $",
+	"$Id$",
 	TRACE_FORMAT_PCAP,
 	NULL,				/* probe filename */
 	NULL,				/* probe magic */
 	pcap_init_input,		/* init_input */
 	pcap_config_input,		/* config_input */
 	pcap_start_input,		/* start_input */
-	NULL,				/* pause_input */
+        pcap_pause_input,		/* pause_input */
 	pcap_init_output,		/* init_output */
 	NULL,				/* config_output */
 	NULL,				/* start_output */
@@ -727,14 +852,17 @@ static struct libtrace_format_t pcap = {
 	pcap_read_packet,		/* read_packet */
 	pcap_prepare_packet,		/* prepare_packet */
 	NULL,				/* fin_packet */
-	pcap_write_packet,		/* write_packet */
+    NULL,                   /* can_hold_packet */
+	pcap_write_packet,		/* write_packet */  
+        pcap_flush_output,              /* flush_output */
 	pcap_get_link_type,		/* get_link_type */
-	pcap_get_direction,		/* get_direction */
+	pcapint_get_direction,		/* get_direction */
 	pcap_set_direction,		/* set_direction */
 	NULL,				/* get_erf_timestamp */
 	pcap_get_timeval,		/* get_timeval */
 	NULL,				/* get_seconds */
 	NULL,				/* get_timespec */
+	NULL,                           /* get_meta_section */
 	NULL,				/* seek_erf */
 	NULL,				/* seek_timeval */
 	NULL,				/* seek_seconds */
@@ -745,16 +873,17 @@ static struct libtrace_format_t pcap = {
 	NULL,				/* get_received_packets */
 	NULL,				/* get_filtered_packets */
 	NULL,				/* get_dropped_packets */
-	NULL,				/* get_captured_packets */
+	NULL,				/* get_statistics */
 	NULL,				/* get_fd */
 	trace_event_trace,		/* trace_event */
 	pcap_help,			/* help */
-	NULL				/* next pointer */
+	NULL,			/* next pointer */
+	NON_PARALLEL(false)
 };
 
 static struct libtrace_format_t pcapint = {
 	"pcapint",
-	"$Id: format_pcap.c 1664 2010-10-06 00:37:07Z salcock $",
+	"$Id$",
 	TRACE_FORMAT_PCAP,
 	NULL,				/* probe filename */
 	NULL,				/* probe magic */
@@ -770,14 +899,17 @@ static struct libtrace_format_t pcapint = {
 	pcap_read_packet,		/* read_packet */
 	pcap_prepare_packet,		/* prepare_packet */
 	NULL,				/* fin_packet */
+    NULL,               /* can_hold_packet */
 	pcapint_write_packet,		/* write_packet */
+	NULL,		                /* flush_output */
 	pcap_get_link_type,		/* get_link_type */
-	pcap_get_direction,		/* get_direction */
+	pcapint_get_direction,		/* get_direction */
 	pcap_set_direction,		/* set_direction */
 	NULL,				/* get_erf_timestamp */
 	pcap_get_timeval,		/* get_timeval */
 	NULL,				/* get_seconds */
 	NULL,				/* get_timespec */
+	NULL,                           /* get_meta_section */
 	NULL,				/* seek_erf */
 	NULL,				/* seek_timeval */
 	NULL,				/* seek_seconds */
@@ -787,12 +919,13 @@ static struct libtrace_format_t pcapint = {
 	pcap_set_capture_length,	/* set_capture_length */
 	NULL,				/* get_received_packets */
 	NULL,				/* get_filtered_packets */
-	pcap_get_dropped_packets,	/* get_dropped_packets */
-	NULL,				/* get_captured_packets */
+	NULL,	                        /* get_dropped_packets */
+	pcap_get_statistics,		/* get_statistics */
 	pcap_get_fd,			/* get_fd */
 	trace_event_device,		/* trace_event */
 	pcapint_help,			/* help */
-	NULL				/* next pointer */
+	NULL,			/* next pointer */
+	NON_PARALLEL(true)
 };
 
 void pcap_constructor(void) {
