@@ -1,42 +1,35 @@
 /*
- * This file is part of libtrace
  *
- * Copyright (c) 2007,2008,2009,2010 The University of Waikato, Hamilton, 
- * New Zealand.
- *
- * Authors: Daniel Lawson 
- *          Perry Lorier
- *          Shane Alcock 
- *          
+ * Copyright (c) 2007-2016 The University of Waikato, Hamilton, New Zealand.
  * All rights reserved.
  *
- * This code has been developed by the University of Waikato WAND 
+ * This file is part of libtrace.
+ *
+ * This code has been developed by the University of Waikato WAND
  * research group. For further information please see http://www.wand.net.nz/
  *
  * libtrace is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * it under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * libtrace is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with libtrace; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
- * $Id: protocols_l3.c 1645 2010-08-02 03:48:16Z salcock $
  *
  */
 
-
 #include "libtrace_int.h"
 #include "libtrace.h"
 #include "protocols.h"
-#include <assert.h>
+#include "checksum.h"
 #include <stdlib.h>
+#include <arpa/inet.h>
 
 #ifdef HAVE_NETPACKET_PACKET_H
 #include <sys/socket.h>
@@ -108,8 +101,11 @@ DLLEXPORT void *trace_get_payload_from_ip(libtrace_ip_t *ipptr, uint8_t *prot,
 {
         void *trans_ptr = 0;
 
-        assert(ipptr != NULL);
-	
+	if (!ipptr) {
+		fprintf(stderr, "NULL libtrace_ip_t pointer passed into trace_get_payload_from_ip()\n");
+		return NULL;
+	}
+
 	/* Er? IPv5? */
 	if (ipptr->ip_v != 4)
 		return NULL;
@@ -125,6 +121,12 @@ DLLEXPORT void *trace_get_payload_from_ip(libtrace_ip_t *ipptr, uint8_t *prot,
 			*remaining = 0;
 			return NULL;
 		}
+		/* If the packet features extra "padding", we probably
+		 * don't want that counting as possible payload, e.g. for
+		 * payload length calculations */
+		//if (*remaining > ntohs(ipptr->ip_len))
+		//	*remaining = ntohs(ipptr->ip_len);
+
 		*remaining-=(ipptr->ip_hl * 4);
 	}
 
@@ -136,13 +138,17 @@ DLLEXPORT void *trace_get_payload_from_ip(libtrace_ip_t *ipptr, uint8_t *prot,
 }
 
 void *trace_get_payload_from_ip6(libtrace_ip6_t *ipptr, uint8_t *prot,
-		uint32_t *remaining) 
-{
+		uint32_t *remaining) {
 	void *payload = (char*)ipptr+sizeof(libtrace_ip6_t);
 	uint8_t nxt;
+	uint16_t len;
 
-	assert (ipptr != NULL);
- 	nxt = ipptr->nxt;	
+	if (!ipptr) {
+		fprintf(stderr, "NULL libtrace_ip6_t passed into trace_get_payload_from_ip6()\n");
+		return NULL;
+	}
+
+ 	nxt = ipptr->nxt;
 	if (remaining) {
 		if (*remaining<sizeof(libtrace_ip6_t)) {
 			*remaining = 0;
@@ -155,14 +161,35 @@ void *trace_get_payload_from_ip6(libtrace_ip6_t *ipptr, uint8_t *prot,
 		switch (nxt) {
 			case 0: /* hop by hop options */
 			case TRACE_IPPROTO_ROUTING:
-			case TRACE_IPPROTO_FRAGMENT:
-			case TRACE_IPPROTO_ESP:
 			case TRACE_IPPROTO_AH: 
 			case TRACE_IPPROTO_DSTOPTS: 
-				{
-					uint16_t len=((libtrace_ip6_ext_t*)payload)->len
-					+sizeof(libtrace_ip6_ext_t);
+			{
+				/* Length does not include the first 8 bytes */
+				len=((libtrace_ip6_ext_t*)payload)->len * 8;
+				len += 8;
 
+
+				if (remaining) {
+					if (*remaining < len) {
+						/* Snap too short */
+						*remaining = 0;
+						return NULL;
+					}
+					*remaining-=len;
+				}
+
+				nxt=((libtrace_ip6_ext_t*)payload)->nxt;
+				payload=(char*)payload+len;
+				continue;
+			}
+			case TRACE_IPPROTO_ESP: 
+			{
+				if (prot) *prot=TRACE_IPPROTO_ESP;
+				return payload;
+			}
+			case TRACE_IPPROTO_FRAGMENT:
+				{
+					len = sizeof(libtrace_ip6_frag_t);
 					if (remaining) {
 						if (*remaining < len) {
 							/* Snap too short */
@@ -171,11 +198,11 @@ void *trace_get_payload_from_ip6(libtrace_ip6_t *ipptr, uint8_t *prot,
 						}
 						*remaining-=len;
 					}
-
+					nxt=((libtrace_ip6_frag_t*)payload)->nxt;
 					payload=(char*)payload+len;
-					nxt=((libtrace_ip6_ext_t*)payload)->nxt;
 					continue;
 				}
+
 			default:
 				if (prot) *prot=nxt;
 				return payload;
@@ -198,20 +225,29 @@ DLLEXPORT void *trace_get_layer3(const libtrace_packet_t *packet,
 	if (!remaining) remaining=&dummy_remaining;
 
 	/* use l3 cache */
-	if (packet->l3_header)
+	if (packet->cached.l3_header)
 	{
+		/*
 		link = trace_get_packet_buffer(packet,&linktype,remaining);
 
 		if (!link)
 			return NULL;
+		*/
 
-		*ethertype = packet->l3_ethertype;
-		*remaining -= (packet->l3_header - link);
+		*ethertype = packet->cached.l3_ethertype;
+		/* *remaining -= (packet->cached.l3_header - link); */
+		*remaining = packet->cached.l3_remaining;
 
-		return packet->l3_header;
+		return packet->cached.l3_header;
 	}
 
-	link = trace_get_layer2(packet,&linktype,remaining);
+        if (packet->cached.l2_header) {
+                link = packet->cached.l2_header;
+                linktype = packet->cached.link_type;
+                *remaining = packet->cached.l2_remaining;
+        } else {
+        	link = trace_get_layer2(packet,&linktype,remaining);
+        }
 	iphdr = trace_get_payload_from_layer2(
 			link,
 			linktype,
@@ -230,7 +266,7 @@ DLLEXPORT void *trace_get_layer3(const libtrace_packet_t *packet,
 			iphdr=trace_get_payload_from_mpls(
 					  iphdr,ethertype,remaining);
 
-			if (iphdr && ethertype == 0x0) {
+			if (iphdr && *ethertype == 0x0) {
 				iphdr=trace_get_payload_from_ethernet(
 						iphdr,ethertype,remaining);
 			}
@@ -251,8 +287,9 @@ DLLEXPORT void *trace_get_layer3(const libtrace_packet_t *packet,
 
 	/* Store values in the cache for later */
 	/* Cast away constness, nasty, but this is just a cache */
-	((libtrace_packet_t*)packet)->l3_ethertype = *ethertype;
-	((libtrace_packet_t*)packet)->l3_header = iphdr;
+	((libtrace_packet_t*)packet)->cached.l3_ethertype = *ethertype;
+	((libtrace_packet_t*)packet)->cached.l3_header = iphdr;
+	((libtrace_packet_t*)packet)->cached.l3_remaining = *remaining;
 
 	return iphdr;
 }
@@ -309,7 +346,61 @@ DLLEXPORT int trace_get_next_option(unsigned char **ptr,int *len,
 				return 0;
 			return 1;
 	}
-	assert(0);
+}
+
+static char *sockaddr_to_string(struct sockaddr *addrptr, char *space,
+		int spacelen) {
+
+	if (!addrptr) {
+		fprintf(stderr, "NULL sockaddr passed into sockaddr_to_string()\n");
+		return NULL;
+	}
+	if (!space) {
+		fprintf(stderr, "NULL buffer space passed into sockaddr_to_string()\n");
+		return NULL;
+	}
+	if (spacelen <= 0) {
+		fprintf(stderr, "Buffer size must be greater than 0 when passed into sockaddr_to_string()\n");
+		return NULL;
+	}
+
+	if (addrptr->sa_family == AF_INET) {
+		struct sockaddr_in *v4 = (struct sockaddr_in *)addrptr;
+		inet_ntop(AF_INET, &(v4->sin_addr), space, spacelen);
+	}
+
+	else if (addrptr->sa_family == AF_INET6) {
+		struct sockaddr_in6 *v6 = (struct sockaddr_in6 *)addrptr;
+		inet_ntop(AF_INET6, &(v6->sin6_addr), space, spacelen);
+	}
+#ifdef HAVE_NETPACKET_PACKET_H
+	else if (addrptr->sa_family == AF_PACKET) {
+		struct sockaddr_ll *l2addr = (struct sockaddr_ll *)addrptr;
+		uint8_t *macbytes = (uint8_t *)l2addr->sll_addr;
+
+		snprintf(space, spacelen, "%02x:%02x:%02x:%02x:%02x:%02x",
+				macbytes[0], macbytes[1], macbytes[2],
+				macbytes[3], macbytes[4], macbytes[5]);
+
+	}
+#else
+	else if (addrptr->sa_family == AF_LINK) {
+		struct sockaddr_dl *l2addr = (struct sockaddr_dl *)addrptr;
+		uint8_t *macbytes = (uint8_t *)l2addr->sdl_data;
+
+		snprintf(space, spacelen, "%02x:%02x:%02x:%02x:%02x:%02x",
+				macbytes[0], macbytes[1], macbytes[2],
+				macbytes[3], macbytes[4], macbytes[5]);
+	
+	}
+#endif
+	else {
+		space[0] = '\0';
+		return NULL;
+	}
+
+	return space;
+
 }
 
 /* Extract the source mac address from a frame and bundle it up into a sockaddr */
@@ -428,6 +519,7 @@ DLLEXPORT struct sockaddr *trace_get_source_address(
 				addr6->sin6_port=0;
 			addr6->sin6_flowinfo=0;
 			addr6->sin6_addr=ip6->ip_src;
+			addr6->sin6_scope_id = 0;
 			return addr;
 		}
 		default:
@@ -436,6 +528,27 @@ DLLEXPORT struct sockaddr *trace_get_source_address(
 }
 
 
+DLLEXPORT char *trace_get_source_address_string(
+		const libtrace_packet_t *packet, char *space, int spacelen) {
+
+	static char staticspace[INET6_ADDRSTRLEN];
+	struct sockaddr_storage addr;
+	struct sockaddr *addrptr;
+	
+
+	if (space == NULL || spacelen == 0) {
+		space = staticspace;
+		spacelen = INET6_ADDRSTRLEN;
+	}
+
+	addrptr = trace_get_source_address(packet, (struct sockaddr *)&addr);
+
+	if (addrptr == NULL)
+		return NULL;
+	
+	return sockaddr_to_string(addrptr, space, spacelen);
+}
+
 static struct sockaddr *get_destination_ethernet_address(
 	libtrace_ether_t *ethernet, struct sockaddr *addr)
 {
@@ -554,4 +667,157 @@ DLLEXPORT struct sockaddr *trace_get_destination_address(
 	}
 }
 
+DLLEXPORT char *trace_get_destination_address_string(
+		const libtrace_packet_t *packet, char *space, int spacelen) {
 
+	struct sockaddr_storage addr;
+	struct sockaddr *addrptr;
+	
+	static char staticspace[INET6_ADDRSTRLEN];
+
+	if (space == NULL || spacelen == 0) {
+		space = staticspace;
+		spacelen = INET6_ADDRSTRLEN;
+	}
+
+	addrptr = trace_get_destination_address(packet, 
+			(struct sockaddr *)&addr);
+
+	if (addrptr == NULL)
+		return NULL;
+	
+	return sockaddr_to_string(addrptr, space, spacelen);
+}
+
+DLLEXPORT uint16_t *trace_checksum_layer3(libtrace_packet_t *packet, 
+		uint16_t *csum) {
+
+	void *l3;
+	uint16_t ethertype;
+	uint32_t remaining;
+	char *csum_ptr;
+
+	uint8_t safety[65536];
+
+	if (csum == NULL)
+		return NULL;
+	
+	l3 = trace_get_layer3(packet, &ethertype, &remaining);
+		
+	if (l3 == NULL)
+		return NULL;
+	
+	if (ethertype == TRACE_ETHERTYPE_IP) {
+		libtrace_ip_t *ip = (libtrace_ip_t *)l3;
+		if (remaining < sizeof(libtrace_ip_t))
+			return NULL;
+
+		csum_ptr = (char *)(&ip->ip_sum);
+
+		/* I hate memcpys, but this is the only truly safe way to
+		 * do this without modifying the packet. I'm trying to be
+		 * careful about not creating any more thread-safety issues
+		 * than there already are :) */
+		memcpy(safety, ip, ip->ip_hl * sizeof(uint32_t));
+		
+		/* Set the checksum to zero, so we can do the calculation */
+		ip = (libtrace_ip_t *)safety;
+		ip->ip_sum = 0;
+
+		*csum = checksum_buffer(safety, ip->ip_hl * sizeof(uint32_t));
+		
+		/* Remember to byteswap appropriately */
+		*csum = ntohs(*csum);
+		
+		return (uint16_t *)csum_ptr;
+	}
+
+	return NULL;
+}
+
+DLLEXPORT uint16_t trace_get_fragment_offset(const libtrace_packet_t *packet, 
+                uint8_t *more) {
+
+        void *l3;
+        uint16_t ethertype;
+        uint32_t remaining;
+
+        *more = 0;
+
+        l3 = trace_get_layer3(packet, &ethertype, &remaining);
+        if (l3 == NULL)
+                return 0;
+
+        if (ethertype == TRACE_ETHERTYPE_IP) {
+                libtrace_ip_t *ip = (libtrace_ip_t *)l3;
+                uint16_t offset = 0;
+
+                /* Fragment offset appears in 7th and 8th bytes */
+                if (remaining < 8)
+                        return 0;
+                 
+                offset = ntohs(ip->ip_off);
+
+                if ((offset & 0x2000) != 0)
+                        *more = 1;
+                return (offset & 0x1FFF) * 8;
+        }
+
+        if (ethertype == TRACE_ETHERTYPE_IPV6) {
+                libtrace_ip6_t *ip6 = (libtrace_ip6_t *)l3;
+                void *payload = ip6 + 1;
+                uint8_t nxt = ip6->nxt;
+                uint16_t len;
+                
+                /* First task, find a Fragment header if present */
+                if (remaining < sizeof(libtrace_ip6_t))
+                        return 0;
+                remaining -= sizeof(libtrace_ip6_t);
+
+                /* Adapted from trace_get_payload_from_ip6 */
+                while (1) {
+                        switch (nxt) {
+                        case 0:
+                        case TRACE_IPPROTO_ROUTING:
+			case TRACE_IPPROTO_AH: 
+			case TRACE_IPPROTO_DSTOPTS: 
+                        {
+
+				/* Length does not include the first 8 bytes */
+				len=((libtrace_ip6_ext_t*)payload)->len * 8;
+				len += 8;
+
+			        if (remaining < len) {
+                                        /* Snap too short */
+                                        return 0;
+                                }
+                                remaining-=len;
+
+				nxt=((libtrace_ip6_ext_t*)payload)->nxt;
+				continue;
+			}
+			case TRACE_IPPROTO_FRAGMENT:
+                        {
+                                libtrace_ip6_frag_t *frag = (libtrace_ip6_frag_t *)payload;
+                                uint16_t offset;
+                                len = sizeof(libtrace_ip6_frag_t);
+                                if (remaining < len) {
+                                        /* Snap too short */
+                                        return 0;
+                                }
+                                remaining-=len;
+
+                                offset = ntohs(frag->frag_off);
+                                if ((offset & 0x0001) != 0) 
+                                        *more = 1;
+
+                                return ((offset & 0xFFF8) >> 3) * 8;
+                         }
+                         default:
+                                return 0;
+                         }
+                }
+
+        }
+        return 0;
+}
